"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AggregateBase = void 0;
const entity_base_1 = require("./entity.base");
class AggregateBase extends entity_base_1.EntityBase {
    constructor(id, props, originalVersion, loaded) {
        super(id, props);
        if (originalVersion < 0)
            throw new Error("Version must be set with non-negative number");
        this._originalVersion = originalVersion;
        this._events = [];
        this._loaded = loaded;
        if (this.isNew()) {
            this.mark();
        }
    }
    static isAggregate(obj) {
        return obj instanceof AggregateBase;
    }
    hasUnpublishedEvents() {
        return Boolean(this.unpublishedEvents.length);
    }
    publishEvents() {
        const events = this.unpublishedEvents.slice();
        this.clearEvents();
        return events;
    }
    recordEvent(event) {
        if (!this.isNew())
            this._events.push(event);
    }
    clearEvents() {
        this._events = [];
    }
    // mean has no change was saved before
    isNew() {
        return this._originalVersion === 0 && !this._loaded;
    }
    get unpublishedEvents() {
        return this._events;
    }
    get originalVersion() {
        return this._originalVersion;
    }
    get version() {
        if (this.hasChanged())
            return this._originalVersion + 1;
        return this._originalVersion;
    }
    get loaded() {
        return this._loaded;
    }
}
exports.AggregateBase = AggregateBase;
