import { Type } from "../types/type";
export type EntityUpdater = () => void;
export declare abstract class EntityBase<P> {
    protected readonly _id: string;
    protected _props: P;
    protected _marked: boolean;
    constructor(id: string, props: P);
    static isEntity(obj: any): obj is AnyEntity;
    abstract validateProps(props: P): void;
    get id(): string;
    private setProps;
    hasId(id: string): boolean;
    protected mark(): void;
    protected unmark(): void;
    protected update(updater: EntityUpdater): void;
    hasChanged(): boolean;
    equalsType(entity: AnyEntity): boolean;
    equals(entity: AnyEntity): boolean;
    getProps(): P;
}
export type AnyEntity = EntityBase<any>;
export type TypeEntity<T extends AnyEntity = AnyEntity> = Type<T>;
export type GetEntityProps<T extends AnyEntity> = T extends EntityBase<infer P> ? P : any;
export type EntityConstructor<T extends AnyEntity = AnyEntity> = new (...args: ConstructorParameters<typeof EntityBase<GetEntityProps<T>>>) => T;
