"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EntityBase = void 0;
const lodash_1 = __importDefault(require("lodash"));
class EntityBase {
    constructor(id, props) {
        this._marked = false;
        if (!id)
            throw new Error("Id must be provided");
        this._id = id;
        this.setProps(props);
    }
    static isEntity(obj) {
        return obj instanceof EntityBase;
    }
    get id() {
        return this._id;
    }
    setProps(props) {
        this.validateProps(props);
        this._props = props;
    }
    hasId(id) {
        return this.id === id;
    }
    mark() {
        this._marked = true;
    }
    unmark() {
        this._marked = false;
    }
    update(updater) {
        updater();
        this.validateProps(this._props);
        this.mark();
    }
    hasChanged() {
        return this._marked;
    }
    equalsType(entity) {
        return entity instanceof this.constructor;
    }
    equals(entity) {
        if (!this.equalsType(entity))
            return false;
        return this.hasId(entity.id);
    }
    getProps() {
        return lodash_1.default.cloneDeep(this._props);
    }
}
exports.EntityBase = EntityBase;
