import { Type } from "../types/type";
export declare abstract class ValueObjectBase<P> {
    protected readonly props: Readonly<P>;
    constructor(props: P);
    static isValueObject(obj: any): boolean;
    abstract validateProps(props: P): void;
    equalsType(obj: ValueObjectBase<P>): boolean;
    equals(obj: ValueObjectBase<P>): boolean;
    cloneWith(props: Partial<P>): this;
    clone(): this;
}
export type AnyValueObject = ValueObjectBase<any>;
export type TypeValueObject<T extends AnyValueObject = AnyValueObject> = Type<T>;
export type GetValueObjectProps<T extends AnyValueObject> = T extends ValueObjectBase<infer P> ? P : any;
export type ValueObjectConstructor<T extends AnyValueObject = AnyValueObject> = new (...args: ConstructorParameters<typeof ValueObjectBase<GetValueObjectProps<T>>>) => T;
